from .download_utilities import DownloadManager
from .examples import (
    download_bracket_dsco,
    download_bracket_fmd,
    download_bracket_scdoc,
    download_deformed_blade_dsco,
    download_deformed_blade_fmd,
    download_deformed_blade_scdoc,
    download_elbow_dsco,
    download_elbow_fmd,
    download_elbow_pmdat,
    download_elbow_scdoc,
    download_f1_rw_drs_stl,
    download_f1_rw_enclosure_stl,
    download_f1_rw_end_plates_stl,
    download_f1_rw_main_plane_stl,
    download_pcb_pmdat,
    download_pcb_scdoc,
    download_pipe_tee_dsco,
    download_pipe_tee_fmd,
    download_pipe_tee_pmdat,
    download_pipe_tee_scdoc,
    download_saddle_bracket_dsco,
    download_saddle_bracket_fmd,
    download_saddle_bracket_scdoc,
    download_toy_car_dsco,
    download_toy_car_fmd,
    download_toy_car_pmdat,
    download_toy_car_scdoc,
    download_turbine_blade_cdb,
)
from .unit_test_examples import download_test_examples
